//***********************************************************************************************
//
//    Copyright (c) 2005 Molecular Devices.
//    All rights reserved.
//
//***********************************************************************************************
// MODULE:  AXDD1550B.HPP
// PURPOSE: Interface definition for AXDD1550B.DLL
// AUTHOR:  BHI  Aug 1998
//

#ifndef INC_AXDD1550B_HPP
#define INC_AXDD1550B_HPP

#pragma pack(push,1)
#include "\AxonDev\Comp\CommonLib\adcdac.h"

const UINT DD1550B_ANY_DEVICE       = UINT(-1);
const UINT DD1550B_MAX_AI_CHANNELS  = 8;            
const UINT DD1550B_MAX_AO_CHANNELS  = 8;            
const UINT DD1550B_MAX_TELEGRAPHS   = 4;
const UINT DD1550B_MAX_DO_CHANNELS  = 16;
const UINT DD1550B_MAX_ANC_CHANNELS = 8;	 
const UINT DD1550B_MAX_HARMONIC_CHNANELS = 2 ;

// Active bits in the digital input stream.
const UINT DD1550B_BIT_EXT_TRIGGER = 0x0001;
const UINT DD1550B_BIT_EXT_TAG     = 0x0002;

struct DD1550B_Info
{
	WORD   wVendorID;
	WORD   wProductID;
	UINT   uSerialNumber;
	char   szName[32];
	char   szFirmwareVersion[16];
	char   szFpgaVersion[16];
	UINT   uInputBufferSamples;
	UINT   uOutputBufferSamples;
	UINT   uAIChannels;
	UINT   uAOChannels;
	UINT   uTelegraphs;
	UINT   uDOChannels;                    // (bits)
	double dMinSequencePeriodUS;
	double dMaxSequencePeriodUS;
	double dSequenceQuantaUS;
	UINT   uMinPrequeueSamples;
	UINT   uMaxPrequeueSamples;
	WORD   wScopeOutBit;
	WORD   wANCGatingBits[3];
	bool   bUSB1;
	unsigned char ucMaxSamplingFreq;       
	unsigned char ucANC_Enable;            

	DD1550B_Info()  
	{
		memset( this, 0, sizeof(*this));
		uSerialNumber = DD1550B_ANY_DEVICE;
	}
};

//========================================================================================
// Constants for the protocol.

const UINT DD1550B_FLAG_EXT_TRIGGER  = 0x0001;
const UINT DD1550B_FLAG_TAG_BIT0     = 0x0002;
const UINT DD1550B_FLAG_STOP_ON_TC   = 0x0004;
const UINT DD1550B_FLAG_SCOPE_OUT    = 0x0008;
const WORD DD1550B_FLAG_DO15_OUT     = 0x8000;



const UINT DD1550B_IO_TIMEOUT  = 1000;

//==============================================================================================
// STRUCTURE: DD1550B_Protocol
// PURPOSE:   Describes acquisition settings.
//
struct DD1550B_Protocol
{
	double            dSequencePeriodUS;      // Sequence interval in us.
	UINT              uFlags;                 // Boolean flags that control options.

	int               nScopeOutAIChannel;     // Analog Input to generate "Scope Output" pulse     
	short             nScopeOutThreshold;     // "Scope Output"=on threshold level  [counts]
	short             nScopeOutHysteresis;    // "Scope Output"=on hysteresis delta [counts]
	bool              bScopeOutPolarity;      // TRUE = positive polarity.

	// Inputs:
	UINT              uAIChannels;
	int               anAIChannels[DD1550B_MAX_AI_CHANNELS];
	MDC_DATABUFFER       *pAIBuffers;
	UINT              uAIBuffers;
	bool              bDIEnable;
	bool              bUnused1[3];            // (alignment padding)

	// Outputs:
	UINT              uAOChannels;
	int               anAOChannels[DD1550B_MAX_AO_CHANNELS];
	MDC_DATABUFFER		  *pAOBuffers;
	UINT              uAOBuffers;
	bool              bDOEnable;
	bool              bUnused2[3];            // (alignment padding)

	UINT              uChunksPerSecond;       // Granularity of data transfer.
	UINT              uTerminalCount;         

	int               anANCEnable[DD1550B_MAX_ANC_CHANNELS];    
	            							                              
	int				  anHarmonics[DD1550B_MAX_HARMONIC_CHNANELS];	
															

	int					uNoOfAvgCycle;       

	bool              bSaveAI;
	bool              bSaveAO;           
	double		      dMinSequencePeriodUS;   
	double		      dMaxSequencePeriodUS;   

	UINT			  nTriggerTimeout;		 

	DD1550B_Protocol()
	{
		memset(this, 0, sizeof(*this));
		uChunksPerSecond = 20;
		nTriggerTimeout = DD1550B_IO_TIMEOUT; 
	}
};

//==============================================================================================
// STRUCTURE: DD1550B_Calibration
// PURPOSE:   Describes calibration constants for data correction.
//
struct DD1550B_Calibration
{
	float anADCGains[DD1550B_MAX_AI_CHANNELS];    // Get/Set 
	short anADCOffsets[DD1550B_MAX_AI_CHANNELS];  // Get/Set

	float afDACGains[DD1550B_MAX_AO_CHANNELS];    // Get
	short anDACOffsets[DD1550B_MAX_AO_CHANNELS];  // Get

	int nMaxSamplingFreq;

	int nNumberOf_ANC_Channel;                     

	DD1550B_Calibration() 
	{
		memset(this, 0, sizeof(*this)); 
		for (UINT i=0; i<ELEMENTS_IN(afDACGains); i++)
			afDACGains[i] = 1.0F;
		for (UINT i=0; i<ELEMENTS_IN(anADCGains); i++)
			anADCGains[i] = 1.0F;
	}
};

//==============================================================================================
// STRUCTURE: DD1550B_PowerOnData
// PURPOSE:   Contains items that are set in the EEPROM of the DD1550B as power-on defaults.
//
struct DD1550B_PowerOnData
{
	UINT  uDigitalOuts;
	short anAnalogOuts[DD1550B_MAX_AO_CHANNELS];

	DD1550B_PowerOnData()
	{
		memset(this, 0, sizeof(*this));
	}
};

//==============================================================================================
// STRUCTURE: Start acquisition info.
// PURPOSE:   To store the start acquisition time and precision, by querying a high resolution 
//            timer before and after the start acquisition SCSI command.
//
struct DD1550B_StartAcqInfo
{
	SYSTEMTIME StartTime;        // Stores the time and date of the begginning of the acquisition. 
	LONGLONG   n64PreStartAcq;   // Stores the high resolution counter before the acquisition start.
	LONGLONG   n64PostStartAcq;  // Stores the high resolution counter after the acquisition start.

	DD1550B_StartAcqInfo() { memset(this, 0, sizeof(*this)); }
};

#pragma pack(pop)

class DD1550B_Device
{
public:    // Public interface
	virtual ~DD1550B_Device() {}

	virtual bool Reset() = 0;

	virtual bool GetDeviceInfo(DD1550B_Info *pInfo) const = 0;
	virtual bool SetSerialNumber(UINT uSerialNumber) = 0;
	virtual bool SetDeviceInfo() =0;
	virtual UINT GetBufferGranularity() const = 0;

	virtual bool SetProtocol(const DD1550B_Protocol *pProtocol) = 0;
	virtual bool GetProtocol(DD1550B_Protocol *pProtocol) const = 0;

	virtual bool StartAcquisition() = 0;
	virtual bool StopAcquisition() = 0;
	virtual bool IsAcquiring() const = 0;

	virtual bool GetAIPosition(LONGLONG &uSequences) = 0;
	virtual bool GetAOPosition(LONGLONG &uSequences) = 0;

	virtual bool GetAIValue(UINT uAIChannel, short &nValue) const = 0;
	virtual bool GetDIValue(WORD &wValue) const = 0;

	virtual bool SetAOValue(UINT uAOChannel, short nValue) = 0;
	virtual bool SetDOValue(WORD wValue) = 0;

	virtual bool SetTrigThreshold(short nValue) = 0;
	virtual bool GetTrigThreshold(short &nValue) const = 0;

	virtual bool ReadTelegraphs(UINT uFirstChannel, short *pnValue, UINT uValues) const = 0;
	virtual void GetTimeAtStartOfAcquisition(DD1550B_StartAcqInfo &StartAcqInfo) const = 0;

	virtual bool GetCalibrationParams(DD1550B_Calibration &Params) const = 0;
	virtual bool SetCalibrationParams(const DD1550B_Calibration &Params) = 0;

	virtual bool GetPowerOnData(DD1550B_PowerOnData &Data) const = 0;
	virtual bool SetPowerOnData(const DD1550B_PowerOnData &Data) = 0;

	virtual bool GetEepromParams(void *pvEepromImage, UINT uBytes) const = 0;
	virtual bool SetEepromParams(const void *pvEepromImage, UINT uBytes) = 0;

	virtual bool GetLastErrorText(char *pszMsg, UINT uMsgLen) const = 0;
	virtual int  GetLastError() const = 0;
};

// Find, Open & close device.
UINT WINAPI DD1550B_CountDevices();
UINT WINAPI DD1550B_FindDevices(DD1550B_Info *pInfo, UINT uMaxDevices, int *pnError);
bool WINAPI DD1550B_GetErrorText(int nError, char *pszMsg, UINT uMsgLen);

DD1550B_Device*  WINAPI DD1550B_OpenDevice(UINT uSerialNumber, int *pnError, bool bIgnoreCRCErr);
void            WINAPI DD1550B_CloseDevice(DD1550B_Device *pDevice);

// Utility functions
DAC_VALUE WINAPI DD1550B_VoltsToDAC(const DD1550B_Calibration &CalData, UINT uDAC, double dVolts);
double    WINAPI DD1550B_DACtoVolts(const DD1550B_Calibration &CalData, UINT uDAC, DAC_VALUE nDAC);
ADC_VALUE WINAPI DD1550B_VoltsToADC(double dVolts);
double    WINAPI DD1550B_ADCtoVolts(ADC_VALUE nADC);

// Error codes
const int DD1550B_ERROR                       = 0x01000000;
const int DD1550B_ERROR_OUTOFMEMORY           = 0x01000002;
const int DD1550B_ERROR_STARTACQ              = 0x01000006;
const int DD1550B_ERROR_STOPACQ               = 0x01000007;
const int DD1550B_ERROR_READDATA              = 0x01000009;
const int DD1550B_ERROR_WRITEDATA             = 0x0100000A;
const int DD1550B_ERROR_THREAD_START          = 0x0100000F;
const int DD1550B_ERROR_THREAD_TIMEOUT        = 0x01000010;
const int DD1550B_ERROR_THREAD_WAIT_ABANDONED = 0x01000011;
const int DD1550B_ERROR_OPEN_RAMWARE          = 0x01000013;
const int DD1550B_ERROR_DOWNLOAD              = 0x01000015; 
const int DD1550B_ERROR_OPEN_FPGA             = 0x01000016; 
const int DD1550B_ERROR_LOAD_FPGA             = 0x01000017; 
const int DD1550B_ERROR_READ_RAMWARE          = 0x01000018;
const int DD1550B_ERROR_SIZE_RAMWARE          = 0x01000019;
const int DD1550B_ERROR_READ_FPGA             = 0x0100001A;
const int DD1550B_ERROR_SIZE_FPGA             = 0x0100001B;
const int DD1550B_ERROR_PIPE_NOT_FOUND        = 0x0100001E; 
const int DD1550B_ERROR_OVERRUN               = 0x01000020; 
const int DD1550B_ERROR_UNDERRUN              = 0x01000021; 
const int DD1550B_ERROR_SETPROTOCOL           = 0x01000022;
const int DD1550B_ERROR_SETAOVALUE            = 0x01000023;
const int DD1550B_ERROR_SETDOVALUE            = 0x01000024;
const int DD1550B_ERROR_GETAIVALUE            = 0x01000025;
const int DD1550B_ERROR_GETDIVALUE            = 0x01000026;
const int DD1550B_ERROR_READTELEGRAPHS        = 0x01000027;
const int DD1550B_ERROR_READCALIBRATION       = 0x01000028;
const int DD1550B_ERROR_WRITECALIBRATION      = 0x01000029;
const int DD1550B_ERROR_READEEPROM            = 0x0100002A;
const int DD1550B_ERROR_WRITEEEPROM           = 0x0100002B;
const int DD1550B_ERROR_SETTHRESHOLD          = 0x0100002C;
const int DD1550B_ERROR_GETTHRESHOLD          = 0x0100002D;
const int DD1550B_ERROR_NOTPRESENT            = 0x0100002E;
const int DD1550B_ERROR_USB1NOTSUPPORTED      = 0x0100002F;
const int DD1550B_ERROR_CRC_CHECKFAIL	     = 0x0100003A;

const int DD1550B_ERROR_DEVICEERROR           = 0x03000000;

const int DD1550B_ERROR_SYSERROR              = 0x02000000;

// All error codes from AXDD1550B.DLL have one of these bits set.
const int DD1550B_ERROR_MASK                  = 0xFF000000;

#endif      // INC_AXDD1550B_HPP
